@@ -141,7 +141,8 @@ librdf_free_hash_datums(librdf_world *world)
   librdf_hash_datum *datum, *next;
   
 #ifdef WITH_THREADS
-  pthread_mutex_lock(world->hash_datums_mutex);
+  if(world->hash_datums_mutex)
+    pthread_mutex_lock(world->hash_datums_mutex);
 #endif
 
   for(datum = world->hash_datums_list; datum; datum = next) {
@@ -151,7 +152,8 @@ librdf_free_hash_datums(librdf_world *world)
   world->hash_datums_list = NULL;
 
 #ifdef WITH_THREADS
-  pthread_mutex_unlock(world->hash_datums_mutex);
+  if(world->hash_datums_mutex)
+    pthread_mutex_unlock(world->hash_datums_mutex);
 #endif
 }
 
@@ -210,6 +212,9 @@ librdf_new_hash_datum(librdf_world *world, void *data, size_t size)
 void
 librdf_free_hash_datum(librdf_hash_datum *datum) 
 {
+  if(!datum)
+    return;
+  
   if(datum->data) {
 #if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
     LIBRDF_DEBUG2("Freeing datum data, %p\n", datum);
@@ -520,6 +525,9 @@ librdf_new_hash_from_hash(librdf_hash* old_hash)
 void
 librdf_free_hash(librdf_hash* hash) 
 {
+  if(!hash)
+    return;
+  
   if(hash->context) {
     if(hash->is_open)
       librdf_hash_close(hash);
@@ -1172,6 +1180,8 @@ librdf_hash_print(librdf_hash* hash, FILE *fh)
 
   key=librdf_new_hash_datum(hash->world, NULL, 0);
   value=librdf_new_hash_datum(hash->world, NULL, 0);
+  if(!key || !value)
+    return;
 
   iterator=librdf_hash_get_all(hash, key, value);
   while(!librdf_iterator_end(iterator)) {
@@ -1182,12 +1192,12 @@ librdf_hash_print(librdf_hash* hash, FILE *fh)
     v=(librdf_hash_datum *)librdf_iterator_get_value(iterator);
     
     fputs("  '", fh);
-    l=fwrite(k->data, k->size, 1, fh);
+    l=fwrite(k->data, 1, k->size, fh);
     if(l != k->size)
       break;
     
     fputs("'=>'", fh);
-    l=fwrite(v->data, v->size, 1, fh);
+    l=fwrite(v->data, 1, v->size, fh);
     if(l != v->size)
       break;
 
@@ -1222,6 +1232,8 @@ librdf_hash_print_keys(librdf_hash* hash, FILE *fh)
   fputs("{\n", fh);
 
   key=librdf_new_hash_datum(hash->world, NULL, 0);
+  if(!key)
+    return;
 
   iterator=librdf_hash_keys(hash, key);
   while(!librdf_iterator_end(iterator)) {
@@ -1229,7 +1241,7 @@ librdf_hash_print_keys(librdf_hash* hash, FILE *fh)
     size_t l;
     
     fputs("  '", fh);
-    l=fwrite(k->data, k->size, 1, fh);
+    l=fwrite(k->data, 1, k->size, fh);
     if(l != k->size)
       break;
     fputs("'\n", fh);
@@ -1282,7 +1294,7 @@ librdf_hash_print_values(librdf_hash* hash, const char *key_string, FILE *fh)
       fputs(", ", fh);
       
     fputc('\'', fh);
-    l=fwrite(v->data, v->size, 1, fh);
+    l=fwrite(v->data, 1, v->size, fh);
     if(l != v->size)
       break;
 
@@ -1535,6 +1547,112 @@ librdf_hash_from_array_of_strings(librdf_hash* hash, const char **array)
 }
 
 
+/**
+ * librdf_hash_to_string:
+ * @hash: #librdf_hash object
+ * @filter: NULL terminated list of keys to ignore
+ *
+ * Format the hash as a string, suitable for parsing by librdf_hash_from_string.
+ *
+ * Note: this method allocates a new string since this is a _to_ method
+ * and the caller must free the resulting memory.
+ *
+ * Return value: string representation of the hash or NULL on failure
+ **/
+char*
+librdf_hash_to_string(librdf_hash* hash, const char *filter[])
+{
+  librdf_hash_datum *key = NULL, *value = NULL;
+  librdf_iterator* iterator = NULL;
+  raptor_stringbuffer* sb;
+  char* result = NULL;
+  size_t len;
+
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(hash, librdf_hash, NULL);
+
+  sb = raptor_new_stringbuffer();
+  if(!sb)
+    goto tidy;
+
+  key=librdf_new_hash_datum(hash->world, NULL, 0);
+  value=librdf_new_hash_datum(hash->world, NULL, 0);
+  if (!key || !value)
+    goto tidy;
+
+  iterator=librdf_hash_get_all(hash, key, value);
+  if (!iterator)
+    goto tidy;
+
+  while(!librdf_iterator_end(iterator)) {
+    librdf_hash_datum *k, *v;
+    int key_is_filtered = 0;
+    size_t i;
+
+    k=(librdf_hash_datum *)librdf_iterator_get_key(iterator);
+    v=(librdf_hash_datum *)librdf_iterator_get_value(iterator);
+    if (!k || !v)
+      break;
+
+    /* Is this one of the keys that we are ignoring? */
+    if(filter) {
+      for(i=0; filter[i]; i++) {
+        size_t f_len = strlen(filter[i]);
+        if(f_len == k->size && strncmp(k->data, filter[i], f_len) == 0) {
+          key_is_filtered = 1;
+          break;
+        }
+      }
+    }
+
+    if(!key_is_filtered) {
+      if(raptor_stringbuffer_length(sb) > 0)
+        raptor_stringbuffer_append_counted_string(sb, (unsigned char*)", ", 2, 1);
+
+      raptor_stringbuffer_append_counted_string(sb, (unsigned char*)k->data, k->size, 1);
+      raptor_stringbuffer_append_counted_string(sb, (unsigned char*)"='", 2, 1);
+
+      for(i=0; i < v->size; i++) {
+        char c = ((char*)v->data)[i];
+        switch (c) {
+          case '\'':
+            raptor_stringbuffer_append_counted_string(sb, (unsigned char*)"\\'", 2, 1);
+            break;
+          case '\\':
+            raptor_stringbuffer_append_counted_string(sb, (unsigned char*)"\\\\", 2, 1);
+            break;
+          default:
+            raptor_stringbuffer_append_counted_string(sb, (unsigned char*)&c, 1, 1);
+            break;
+        }
+      }
+
+      raptor_stringbuffer_append_counted_string(sb, (unsigned char*)"'", 1, 1);
+    }
+
+    librdf_iterator_next(iterator);
+  }
+
+  /* Generate a string result */
+  len=raptor_stringbuffer_length(sb);
+  result=LIBRDF_MALLOC(cstring, len + 1);
+  if (result)
+    raptor_stringbuffer_copy_to_string(sb, (unsigned char*)result, len);
+
+
+  tidy:
+  if(iterator)
+    librdf_free_iterator(iterator);
+  if(value)
+    librdf_free_hash_datum(value);
+  if(key)
+    librdf_free_hash_datum(key);
+  if(sb)
+    raptor_free_stringbuffer(sb);
+
+  return result;
+}
+
+
 /**
  * librdf_hash_get_as_boolean:
  * @hash: #librdf_hash object
@@ -1613,6 +1731,7 @@ librdf_hash_get_as_long (librdf_hash* hash, const char *key)
   return lvalue;
 }
 
+
 /**
  * librdf_hash_put_strings:
  * @hash: hash object
@@ -1773,12 +1892,14 @@ main(int argc, char *argv[])
   const char *test_hash_delete_key="size";
   const unsigned char* template_string=(const unsigned char*)"the shape is %{shape} and the sides are %{sides} created by %{rubik}";
   const unsigned char* template_expected=(const unsigned char*)"the shape is cube and the sides are 6 created by ";
+  const char * filter_string[] = {"field1", NULL};
   int i,j;
   const char *type;
   librdf_hash_datum hd_key, hd_value; /* on stack */
   const char *program=librdf_basename((const char*)argv[0]);
   int b;
   long l;
+  char* string_result;
   unsigned char* template_result;
   librdf_world *world;
   
@@ -1933,14 +2054,14 @@ main(int argc, char *argv[])
     librdf_free_hash_datum(key_hd);
   }
 
-
   fprintf(stdout, "%s: Freeing hash\n", program);
   /* close() done automatically by free so not required */
   /* librdf_hash_close(h2); */
   librdf_free_hash(h2);
 
+
   h2=librdf_new_hash(world, NULL);
-  fprintf(stdout, "%s: Initialising hash from string >>%s<<\n", program, 
+  fprintf(stdout, "%s: Initialising hash from string >>%s<<\n", program,
           test_hash_string);
   librdf_hash_from_string (h2, test_hash_string);
   fprintf(stdout, "%s: resulting ", program);
@@ -1948,9 +2069,50 @@ main(int argc, char *argv[])
   fputc('\n', stdout);
   fprintf(stdout, "%s: values count %d\n", program, librdf_hash_values_count(h2));
 
+  fprintf(stdout, "%s: Converting hash back to a string\n", program);
+  string_result=librdf_hash_to_string(h2, NULL);
+
+  /* Order is not guaranteed, so sadly we can't just do a full string comparison */
+  if(!strstr(string_result, "field1='value1'")) {
+    fprintf(stdout, "%s: Did not see field1='value1' in the generated string >>%s<<\n",
+            program, string_result);
+    exit(1);
+  } else if(!strstr(string_result, "field2='\\'value2'")) {
+    fprintf(stdout, "%s: Did not see field2='\\'value2'' in the generated string >>%s<<\n",
+            program, string_result);
+    exit(1);
+  } else if(!strstr(string_result, "field3='\\\\'")) {
+    fprintf(stdout, "%s: Did not see field3='\\\\' in the generated string >>%s<<\n",
+            program, string_result);
+    exit(1);
+  } else if(!strstr(string_result, "field4='\\\\\\'")) {
+    fprintf(stdout, "%s: Did not see field4='\\\\\\' in the generated string >>%s<<\n",
+            program, string_result);
+    exit(1);
+  } else if(!strstr(string_result, "field5='a'")) {
+    fprintf(stdout, "%s: Did not see field5='a' in the generated string >>%s<<\n",
+            program, string_result);
+    exit(1);
+  } else {
+    fprintf(stdout, "%s: resulting in >>%s<<\n", program, string_result);
+  }
+  free(string_result);
+
+  fprintf(stdout, "%s: Converting hash back to a string with filter\n", program);
+  string_result=librdf_hash_to_string(h2, filter_string);
+  if(strstr(string_result, "field1")) {
+    fprintf(stdout, "%s: Was not expecting >>field1<< to be in the generated string >>%s<<\n",
+            program, string_result);
+    exit(1);
+  } else {
+    fprintf(stdout, "%s: resulting in >>%s<<\n", program, string_result);
+  }
+  free(string_result);
+
   librdf_free_hash(h2);
 
-   
+
+
   fprintf(stdout, "%s: Subtituting into template >>%s<<\n", program, 
           template_string);
   h2=librdf_new_hash(world, NULL);
